#The goal of this script is to project all the NCPM roads data into the same equidistant projection and clip to NCPM counties

##Import necessary modules
import arcpy
import os

arcpy.env.overwriteOutput = True
env = "X:/GIS/Projects/SLR_National_Model/Analysis/2019a Nuisance Flooding/"

#PARAMETERS
origID = "OrigRd_ID"
segID = "Seg_ID"
shp = ".shp"
hpmsData = "Original GIS Files/HPMS 2016"
outData = "IEC GIS/State Runs"
spatRef = "USA Contiguous Equidistant Conic"
countyData = "IEC GIS/Shapefiles/Counties_in_model.shp"

hpms = env + hpmsData #create file path of HPMS 2016 data
path = env + outData #create file path for output data
coastCnt = env + countyData
sr = arcpy.SpatialReference(spatRef)#establish projected coordinate system

#initialize vector of coastal states
#coastSt = ['alabama','california','connecticut','delaware','district','florida','georgia','hawaii','louisiana','maine','maryland','massachusetts','mississippi','newhampshire','newjersey','newyork','northcarolina','oregon','pennsylvania','rhodeisland','southcarolina','texas','virginia','washington']
#no intersections created for hawaii
coastSt =['california']

#loop over all states
for state in coastSt:
    print state
    roadShp = hpms + "/" + state + "2016/" + str.title(state) + ".shp" #create loop over all hpms 2016 data
    statePath = path + "/"+ state #create state folder path    

    #clip to coastal counties shapefile
##    print ("Clipping roads to NCPM counties")
    clipShp = statePath + "/" + state + "_NCPMclip" + shp
##    arcpy.Clip_analysis(roadShp, coastCnt, clipShp)
    
    #project into equal distant coordinate system
    print("1. Projecting into equidistant coordinate system")
    arcpy.AddMessage("Projecting into equidistant coordinate system")
    projectShp = statePath + "/" + state + "_project" + shp
    arcpy.Project_management(clipShp,projectShp, sr)
        
    #create ID field for original road segments
    print("2. Add OrigID Field")
    arcpy.AddField_management(projectShp, origID, "Text")
    print("3. Create OrigID")
    arcpy.AddMessage("Create OrigID")
    arcpy.CalculateField_management(projectShp, origID, '[FID]+1&"OR"') 

    #select layer by structure_ = 0
    noStruc = statePath + "/rd_nostruc" + shp
    arcpy.AddMessage("Selecting roads with STRUCTURE = 0")
    print("4. Selecting roads with STRUCTURE = 0")
    arcpy.MakeFeatureLayer_management(projectShp, "selectStruc", '"STRUCTURE_" = 0')
    arcpy.CopyFeatures_management("selectStruc", noStruc)
