#Runs geoprocessing of Functional Classes 1-2 across coastal states
#Georgia, Louisiana, and Cali use data from 2015 and use script 1a.

#import necessary modules
import arcpy
import os

arcpy.env.overwriteOutput = True
env = "X:/GIS/Projects/SLR_National_Model/Analysis/2019a Nuisance Flooding/"
arcpy.ImportToolbox("C:/Program Files (x86)/ET SpatialTechniques/ET GeoWizards 11.4 for ArcGis 10.4/ET GeoWizards.tbx")

#PARAMETERS
origID = "OrigRd_ID"
segID = "Seg_ID"
shp = ".shp"
hpmsData = "Original GIS Files/HPMS 2016"
outData = "IEC GIS/State Runs"
spatRef = "USA Contiguous Equidistant Conic"
countyData = "IEC GIS/Shapefiles/Counties_in_model.shp"

hpms = env + hpmsData #create file path of HPMS 2016 data
path = env + outData #create file path for output data
coastCnt = env + countyData #create file path of counties in model
sr = arcpy.SpatialReference(spatRef)#establish projected coordinate system

#initialize vector of coastal states
coastSt = ['alabama','connecticut','delaware','district','florida','hawaii','maine','maryland','massachusetts','mississippi','newhampshire','newjersey','newyork','northcarolina','oregon','pennsylvania','rhodeisland','southcarolina','texas','virginia','washington']

#loop over all states
for state in coastSt:
    print state
    roadShp = hpms + "/" + state + "2016/" + str.title(state) + ".shp" #create loop over all hpms 2016 data
    statePath = path + "/"+ state + "_1_2"#create state folder path    

    #clipped to coastal counties shp
    clipShp = statePath + "/" + state + "_NCPMclip" + shp
    
    #projected shp
    projectShp = statePath + "/" + state + "_project" + shp
        
    #select layer by structure_ = 0
    noStruc = statePath + "/rd_nostruc" + shp

    #select layer by ramps
    ramp = statePath + "/ramps" + shp
    arcpy.AddMessage("Selecting ramps")
    arcpy.MakeFeatureLayer_management(projectShp, "select_ramp", '"FACILITY_T" = 4')
    arcpy.CopyFeatures_management("select_ramp", ramp)

    #select layer by FC 1_2 and nonFT6
    fc1_2 = statePath + "/FC1_2" + shp
    arcpy.AddMessage("Selecting fc 1_2")
    arcpy.MakeFeatureLayer_management(projectShp, "select1_2", '"F_SYSTEM" <= 2 AND "F_SYSTEM" <> 0 AND "FACILITY_T" < 6')
    arcpy.CopyFeatures_management("select1_2", fc1_2)    

    #dissolve road segments and keep AADT values and FC, FT
    print ("Dissolve all road segments")
    fcDis = statePath + "/FC1_2Diss" + shp
    arcpy.Dissolve_management(fc1_2, fcDis, "", "", "", "DISSOLVE_LINES")

    #intersect FC1-2 roads with ramps
    print("Creating FC 1-2 ramps")
    ptIntersect = statePath + "/FC1_2_RampInt" + shp
    arcpy.ET_GPPointIntersection(fcDis, ramp, ptIntersect)

    #split dissolved polyline with intersection layer
    print("Split FC1-2 by Ramps")
    splitDis = statePath + "/FC1_2Diss_Split" + shp
    arcpy.ET_GPSplitPolylinesWithFeatureClass(fcDis, ptIntersect, splitDis, 0.5,"")

    #create segID    
    arcpy.AddField_management(splitDis, segID, "Text")
    print("Adding Dissolve Seg IDs")
    arcpy.CalculateField_management(splitDis, segID, '[FID]+1&"SG12"')

    #calculate length of defined segments
    arcpy.AddMessage("Calculate Length of Dissolve Segs")
    arcpy.AddGeometryAttributes_management(splitDis, "LENGTH", "FEET_US","","")

    #split original roads with intersection layer
    print("Split Orig Rds by Ramps")
    splitOrig = statePath + "/FC1_2Orig_RampSplit" + shp
    arcpy.ET_GPSplitPolylinesWithFeatureClass(fc1_2, ptIntersect, splitOrig, 0.5,"")

    #calculate length of split original segments
    print("Calculate Length of Split Rds")
    arcpy.AddGeometryAttributes_management(splitOrig, "LENGTH", "FEET_US","","")

    #calculate VMT
    arcpy.AddField_management(splitOrig, "VMT", "FLOAT")
    print("Calculate VMT")
    arcpy.CalculateField_management(splitOrig, "VMT", "[LENGTH]*[AADT]")

    #Remove bridges from analysis
    splitBTC = statePath + "/FC1_2Orig_RampSplit_NoBTC" + shp
    arcpy.MakeFeatureLayer_management(splitOrig, "removeBTC", '"STRUCTURE_"=0')
    arcpy.CopyFeatures_management("removeBTC", splitBTC)    
    
    #Spatial join OrigSplit segments with DissSeg
    print("Spatial join dissolve segments to split rds")
    joinSplit = statePath + "/FC1_2Split_JoinSeg" + shp
    arcpy.SpatialJoin_analysis(splitBTC, splitDis, joinSplit, "JOIN_ONE_TO_ONE", "", "", "HAVE_THEIR_CENTER_IN", "", "")

    #Dissolve OrigSplit segs by DissSeg and FC
    print("Dissolve split segments by dissolve segment IDs")
    arcpy.AddMessage("Dissolve split segments by dissolve segment IDs")
    weightDiss = statePath + "/FC1_2_WAADT_Diss" + shp
    dissField = ["Seg_ID","LENGTH_1","F_SYSTEM"]
    statField = [["VMT","SUM"]]
    arcpy.Dissolve_management(joinSplit, weightDiss, dissField, statField,"","DISSOLVE_LINES")

    #calculate weighted AADT
    print("Calculate Weighted AADT")
    arcpy.AddField_management(weightDiss, "W_AADT", "FLOAT")
    #add selection criteria to where [LENGTH] <> 0
    arcpy.MakeFeatureLayer_management(weightDiss, "selectLen", '"LENGTH_1" <> 0')
    arcpy.CalculateField_management("selectLen", "W_AADT", "[SUM_VMT]/[LENGTH_1]")

    #remove all layers from data frame
    mxd = arcpy.mapping.MapDocument("CURRENT")
    df = arcpy.mapping.ListDataFrames(mxd)[0]
    for lyr in arcpy.mapping.ListLayers(mxd,"",df):
        arcpy.mapping.RemoveLayer(df,lyr)
