#runs FC 1-2 analysis for nuisance flooding
#uses 2015 data for Georgia, Massachusetts and California
#Georgia bridges were compiled by hand -- merged together data from 2015 and 2016

import arcpy
import os

arcpy.env.overwriteOutput = True
env = "X:/GIS/Projects/SLR_National_Model/Analysis/2019a Nuisance Flooding/"

arcpy.ImportToolbox("C:\Program Files (x86)\ET SpatialTechniques\ET GeoWizards 11.4 for ArcGis 10.4\ET GeoWizards.tbx")
ETGWPath = "C:\Program Files\ETSpatialTechniques\ETGeoWizards\ETGWRun.exe"

#PARAMETERS
origID = "OrigRd_ID"
segID = "Seg_ID"
shp = ".shp"
hpmsData = "Original GIS Files/HPMS 2015"
outData = "IEC GIS/State Runs"
spatRef = "USA Contiguous Equidistant Conic"
countyData = "IEC GIS/Shapefiles/Counties_in_model.shp"

hpms = env + hpmsData #create file path of HPMS 2015 data
path = env + outData #create file path for output data
coastCnt = env + countyData
sr = arcpy.SpatialReference(spatRef)#establish projected coordinate system

#initialize vector of coastal states
coastSt =['california','massachusetts', 'georgia']

#loop over all states
for state in coastSt:
    print state
    roadShp = hpms + "/" + state + "2015/" + str.title(state) + "_Sections.shp" #create loop over all hpms 2015 data
    statePath = path + "/"+ state + "_3_7" #create state folder 3-7 path    
    #clip to coastal counties shapefile
    #print ("Clipping roads to NCPM counties")
    clipShp = statePath + "/" + state + "_NCPMclip" + shp
    arcpy.Clip_analysis(roadShp, coastCnt, clipShp)
    
    #project into equal distant coordinate system
    print("1. Projecting into equidistant coordinate system")
    arcpy.AddMessage("Projecting into equidistant coordinate system")
    projectShp = statePath + "/" + state + "_project" + shp
    arcpy.Project_management(clipShp,projectShp, sr)
        
    #create ID field for original road segments
    print("2. Add OrigID Field")
    arcpy.AddField_management(projectShp, origID, "Text")
    print("3. Create OrigID")
    arcpy.AddMessage("Create OrigID")
    arcpy.CalculateField_management(projectShp, origID, '[FID]+1&"OR"') 

    #select layer by structure_ = 0
    fc3_7 = statePath + "/FC3_7" + shp
    arcpy.AddMessage("Selecting roads with STRUCTURE = 0")
    print("4. Selecting roads with STRUCTURE = 0")
    arcpy.MakeFeatureLayer_management(projectShp, "selectStruc", '"STRUCTURE_" = 0')
    arcpy.CopyFeatures_management("selectStruc", noStruc)

    #select layer by FC 3 - 7
    arcpy.AddMessage("Selecting FC 3-7 Roads")
    print("5. Selecting FC 3-7 Roads")
    arcpy.MakeFeatureLayer_management(projectShp, "select3_7", '"F_SYSTEM" >= 3')
    arcpy.CopyFeatures_management("select3_7", fc3_7)

    #dissolve road segments and keep AADT values and FC, FT
    print ("6. Dissolve all road segments")
    fcDis = statePath + "/FC3_7Diss" + shp
    fcDisCopy = statePath + "/FC3_7DissCopy" + shp
    arcpy.Dissolve_management(fc3_7, fcDis, "", "", "", "DISSOLVE_LINES")
    #make copy of dissolve features for intersection
    arcpy.CopyFeatures_management(fcDis, fcDisCopy)

    #intersect FC3-7 roads with themselves
    print("7. Creating FC 3-7 Intersections")
    arcpy.AddMessage("Creating FC 3-7 Intersections")
    ptIntersect = statePath + "/FC3_7Intersect" + shp
    arcpy.ET_GPPointIntersection(fcDis, fcDisCopy,ptIntersect)

    #split dissolved polyline with intersection layer
    print("8. Split FC3-7 by Intersections")
    arcpy.AddMessage("Split FC3-7 by Intersections")
    splitDis = statePath + "/FC3_7Diss_Split" + shp
    arcpy.ET_GPSplitPolylinesWithFeatureClass(fcDis, ptIntersect, splitDis, 0.5,"")

    #create segID    
    print("9. Adding Dissolve Seg IDs")
    arcpy.AddMessage("Adding Dissolve Seg IDs")
    arcpy.AddField_management(splitDis, segID, "Text")
    arcpy.CalculateField_management(splitDis, segID, '[FID]+1&"SG"')

    #calculate length of defined segments
    arcpy.AddMessage("10. Calculate Length of Dissolve Segs")
    arcpy.AddGeometryAttributes_management(splitDis, "LENGTH", "FEET_US","","")

    #split original roads with intersection layer
    print("11. Split Orig Rds by Intersections")
    arcpy.AddMessage("Split Orig Rds by Intersections")
    splitOrig = statePath + "/FC3_7Orig_Split" + shp
    arcpy.ET_GPSplitPolylinesWithFeatureClass(fc3_7, ptIntersect, splitOrig, 0.5,"")

    #calculate length of split original segments
    print("12. Calculate Length of Split Rds")
    arcpy.AddMessage("Calculate Length of Split Rds")
    arcpy.AddGeometryAttributes_management(splitOrig, "LENGTH", "FEET_US","","")

    #calculate VMT
    arcpy.AddMessage("Calculate VMT")
    arcpy.AddField_management(splitOrig, "VMT", "FLOAT")
    print("13. Calculate VMT")
    arcpy.CalculateField_management(splitOrig, "VMT", "[LENGTH]*[AADT]")

    #remove BTC
    splitBTC = statePath + "/FC3_7Orig_IntSplit_NoBTC" + shp
    arcpy.MakeFeatureLayer_management(splitOrig, "removeBTC", '"STRUCTURE_"=0')
    arcpy.CopyFeatures_management("removeBTC", splitBTC)    

    #Spatial join OrigSplit segments with DissSeg
    print("14. Spatial join dissolve segments to split rds")
    arcpy.AddMessage("Spatial join dissolve segments to split rds")
    joinSplit = statePath + "/FC3_7Split_JoinSeg" + shp
    arcpy.SpatialJoin_analysis(splitBTC, splitDis, joinSplit, "JOIN_ONE_TO_ONE", "", "", "HAVE_THEIR_CENTER_IN", "", "")

    #Dissolve OrigSplit segs by DissSeg and FC
    print("15. Dissolve split segments by dissolve segment IDs")
    arcpy.AddMessage("Dissolve split segments by dissolve segment IDs")
    weightDiss = statePath + "/FC3_7_WAADT_Diss" + shp
    dissField = ["Seg_ID","LENGTH_1","F_SYSTEM"]
    statField = [["VMT","SUM"]]
    arcpy.Dissolve_management(joinSplit, weightDiss, dissField, statField,"","DISSOLVE_LINES")

    #calculate weighted AADT
    print("16. Calculate Weighted AADT")
    arcpy.AddMessage("Calculate Weighted AADT")
    arcpy.AddField_management(weightDiss, "W_AADT", "FLOAT")
    #add selection criteria to where [LENGTH] <> 0
    arcpy.MakeFeatureLayer_management(weightDiss, "selectLen", '"LENGTH_1" <> 0')
    arcpy.CalculateField_management("selectLen", "W_AADT", "[SUM_VMT]/[LENGTH_1]")

    #remove all layers from data frame
    mxd = arcpy.mapping.MapDocument("CURRENT")
    df = arcpy.mapping.ListDataFrames(mxd)[0]
    for lyr in arcpy.mapping.ListLayers(mxd,"",df):
        arcpy.mapping.RemoveLayer(df,lyr)
        
    