%% Disclaimer
%{

The information in this document is currently being subjected to peer 
review. Mention of trade names or commercial products does not constitute 
endorsement or recommendation for use by the developers.

The code here must be used at the user's own risk. 

Neither Industrial Economics, Inc., nor the program authors can assume 
responsibility for model operation, output, interpretation or usage.

The creators of this program have used their best efforts in preparing this 
code. It is not absolutely guaranteed to be error free. The author/programmer 
makes no warrantees, expressed or implied, including without limitation 
warrantees of merchantability or fitness for any particular purpose. No 
liability is accepted in any event for any damages, including accidental or 
consequential damages, lost of profits, costs of lost data or programming 
materials, or otherwise in connection with or arising out of the use of 
this program.

%}

% Tide Gauge Data Processing
%  - build "hoursAtLevel.mat"

clear

filename = 'Projected_1920_2100_HTfloods_hrs.xls'; % (from William Sweet)

names.SLRscen = {'low' 'intlow' 'int' 'inthigh' 'high' 'extreme'};
num.SLRscen = length(names.SLRscen);

warning off
info = readtable(filename,'sheet','Names');
info = info(info.Include==1,:);
warning on

levels = -6:.001:2;

num.gauge = height(info);

for gauge = 1:num.gauge
  
  gaugeID = num2str(info.NOAAID(gauge));
  fn = [gaugeID '_19yr_kopp.mat'];
  load(['from B Sweet\observations\' fn]) % (files from William Sweet)
  
  CDF = (24*365)*(1-pdf_cdf.cdf_hr);
  
  plot(pdf_cdf.freq_hr-info.Minor_Flood_Level(gauge),CDF)
  
  hrsAtLevel(:,gauge) = interp1(pdf_cdf.freq_hr-info.Minor_Flood_Level(gauge),CDF,levels);
  ind = findClosestInd(pdf_cdf.freq_hr,info.Minor_Flood_Level(gauge));
  hrs_at_zero(gauge) = CDF(ind);
  
  for slr = 1:num.SLRscen
    hoursProj(:,slr,gauge) = pdf_cdf.([names.SLRscen{slr} '_hr'])*24*365;
  end
  
  past(:,gauge) = pdf_cdf.past_hr*24*365;
  trend(:,gauge) = pdf_cdf.trend_hr*24*365;
  
end

hrsAtLevel(isnan(hrsAtLevel)) = 8760; % set nans to max hours (only happens on upper end)
scatter(hrs_at_zero,squeeze(hoursProj(1,1,:)))
plot(levels,hrsAtLevel)

key.info = info;
key.levelUnits = 'meters';
key.hrsAtLevelUnits = 'Hours above the minor level at specified SLR levels (every mm)';
save('HoursAtLevel.mat','levels','hrsAtLevel','key')


